% Bienenstock Cooper Munro (BCM Rule)
% input is images, which should have #rows = #image size
% and #columns = #images. For example, 1002 images at 100x100
% resolution should be passed in a matrix of size 1002x10000
% Based of code written by Elie Bienenstock

% Alpha, beta, and p are parameters of the neural network.
% Alpha is the weight-change rate
% Beta is the threshold-change rate
% p is the exponent used in threshold rule, try values from 1.1 to 2
% ite is the number of iterations

function w = NEUR1680_BCM(images,numComponents,alpha,beta,p,ite)
% get images from argument
P = images';

% code for mean centering
%P = images;
%P_mean_face = repmat(mean(P,2),1,1002);
%P = P-P_mean_face; P = P';

% get dimensions of images
n_images   = size(P,1);
size_image = size(P,2);
% initial threshold
theta_v = ones(numComponents);
% initial weights
w = 2e-4*rand(size_image,numComponents);
% variable holds weights as they change over time
% W = 2e-4*rand(size_image,numComponents,ite);
% initialize output
V = zeros(numComponents,ite);
% initialize threshold
THETA_V = zeros(numComponents,ite);
% loop through number of iterations
tic
for t = 1:ite
    % get random image
    u = P(randi(n_images),:);
    for j=1:numComponents
        % calculate response
        v = u*w(:,j);
        % if its the first component don't subtract anything
        if j==1
            % update the weights based on hyperbolic tangent
            w(:,j) = w(:,j) + alpha*tanh(v*(v-theta_v(j)))*u';
            % update the threshold
            theta_v(j) = theta_v(j) + beta*(abs(v)^p - theta_v(j));
        else
            % update the weights based on hyperbolic tangent
            % plus Sanger's rule, subtracting previous components
            modV = v - sum(w(:,1:j-1)*V(1:j-1,t),1);
            w(:,j) = w(:,j) + alpha*tanh(modV*(modV-theta_v(j)))*u';
            % update the threshold
            theta_v(j) = theta_v(j) + beta*(abs(modV)^p - theta_v(j));
        end        
        % set output
        V(j,t) = v;
        % set threshold
        THETA_V(j,t) = theta_v(j);
    end
    % store weights
%     W(:,:,t) = w;
end
toc
end