clear all, close all, clc
% load images
load('images.mat');
% load eigenvectors
load('eigenvectors.mat');
% only keep first 100 eigenvectors
E=E(:,1:100);
% put images in right shape
images = P';
% clear P for more free memory
clear P;
% run GHA on images
W = runGHA(images,4,0.1,0.8,10);

%% Create movie
errors = cell(size(W,2),1);
% create plot
h=figure;
pause(0.01);
jf=get(h,'JavaFrame');
set(jf,'Maximized',1);
pause(0.5);
winsize = get(h,'Position');
% set number of frames
numFrames = 400;
% get iterations to loop through
iters = round(linspace(1,size(W,3),numFrames));

% create video writer object
writerObj = VideoWriter('Figure2.avi');
writerObj.FrameRate = 22;
open(writerObj);

% loop through iterations
for i=1:length(iters)
    % loop through the number of components calculated
    for j=1:size(W,2)
        % calculate error and closest image
        [error,closestImage] = calculateError(W(:,j,iters(i)),E(:,j));
        % store errors
        errors{j} = horzcat(errors{j},error);
        % show the evolved eigenvector
        subplot(2,size(W,2),j);
        imagesc(reshape(closestImage,100,100));
        colormap bone;
        % show evolving error
        subplot(2,size(W,2),size(W,2)+j);
        semilogy(iters(1:i),errors{j});
    end
    % get the frame
    frame=getframe(h,winsize);
    % write frame to video
    writeVideo(writerObj,frame);
end

% generate extra frames comparing the evolved and actual eigenvectors
extraFrames = 50;
for i=1:extraFrames
    for j=1:size(W,2)
        subplot(2,size(W,2),size(W,2)+j);
        imagesc(reshape(E(:,j),[100,100]));
        colormap bone;
    end
    % write frames to video
    frame=getframe(h,winsize);
    writeVideo(writerObj,frame);
end
% close video writer
close(writerObj);
% save still picture
print(h,'-djpeg','Figure1');
% close the figure
close all

%% Run BCM
clear all, close all, clc
% load images
load('images.mat');
% load eigenvectors
load('eigenvectors.mat');
% only keep first 100 eigenvectors
E=E(:,1:100);
% put images in right shape
images = P';
% clear P for more free memory
clear P;
% set number of components to calculate
numComponents = 4;
% run GHA on images
W = runBCM(images,numComponents,1e-6,1e-2,1.5,1e4);
figure
for i=1:numComponents
    subplot(2,2,i);    
    imagesc(reshape(W(:,i,end),[100,100]));
    colormap bone;
end