% Generalized Hebbian Algorithm
% input is images, which should have #rows = #image size
% and #columns = #images. For example, 1002 images at 100x100
% resolution should be passed in a matrix of size 1002x10000

% tauW, tauB, and alpha are parameters of the neural network
% useful parameters are tauW=15000, alpha=0.1, and learningRate = 0.8

function W = runGHA(images,numComponents,tauW,alpha,learningRate)
% get images from argument
P=images;
% tile matrix with duplicates to allow more time for components to converge
P = repmat(P,1,10);
% permute the order of the images
inds=randperm(size(P,2));
P = P(:,inds);
% calculate the mean face of the distribution
P_mean_face = mean(P,2);
% subtract the mean face from the distribution
P = bsxfun(@minus,P,P_mean_face);

% initialize the time constant for weights
tau_w = tauW;
% get number of iterations = number of images
N     = size(P,2);
% initialize number of components
p     = numComponents;
% get image size
m     = size(P,1);
% initialize learning parameters
alph  = alpha;
eta   = learningRate;

% X is input
X       = P;
% W is weight matrix at times t = 1,2, ... ite = # inputs
W       = zeros(m,p,N);
% Y is response
Y       = zeros(m,p);
% initial weight vectors is picked at random in large square
w       = 10*rand(m,p) - 5*ones(m,p);

tic
% loop through images
for n = 1:N
    % input
    x       = X(:,n);
    % for number of outputs/components
    for j=1:p
        % output for component
        y = w(:,j)'*x;
        % if its the first component don't subtract anything
        if j==1
            % first component = Oja's rule
            w(:,j) = w(:,j) + eta*y*x/tau_w - alph*y^2*w(:,j)/tau_w;
        else
            % next components = Sanger's rule
            w(:,j) = w(:,j) + eta*y*(x - sum(w(:,1:j-1).*Y(:,1:j-1),2))...
                /tau_w - y^2*w(:,j)/tau_w;
        end
        % store output
        Y(:,j) = y;
    end
    % store weights
    W(:,:,n) = w;
end
toc
end